package com.link.core.basic.service;

import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.link.core.basic.dao.mapper.BasicMapper;
import com.link.core.basic.dao.siebel.mapper.BasicMapperSiebel;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.query.ExportColumn;
import com.link.core.basic.query.Filter;
import com.link.core.modules.export.ExportWriter;
import com.link.core.modules.export.utils.LinkStdCsvStreamResultHandler;
import com.link.core.modules.export.utils.LinkStdExcelStreamResultHandler;
import com.link.core.modules.export.writer.AbstractFileWriter;
import com.link.core.modules.export.writer.CsvWriter;
import com.link.core.util.CheckUpdAndDelUtil;
import com.link.core.util.ReflectUtil;
import com.link.core.util.StringUtils;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.annotation.Resource;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.transaction.annotation.Transactional;

public class BasicServiceImpl<T extends BasicModel> implements BasicService<T> {
   private static final Logger logger = LogManager.getLogger(BasicServiceImpl.class);
   private Class<T> modelClazz = this.getEntityClazz(this.getClass());
   @Resource
   protected KeyGenerateService keyGenerateService;

   public BasicServiceImpl() {
      if (this.modelClazz == null) {
         throw new BasicServiceException("初始化" + this.getClass() + "时，获取当前泛型T的真实类型失败，modelClazz为空");
      }
   }

   public BasicMapper<T> getBasicMapper() throws Exception {
      throw new BasicServiceException("[" + this.modelClazz.getSimpleName() + "]获取Basic Mapper 失败，请确认程序是否正确！");
   }

   public BasicMapperSiebel<T> getSiebelBasicMapper() throws Exception {
      return null;
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void insert(T entity) throws Exception {
      this.beforInsert(entity);
      this.beforUpsert(entity);
      this.postDefaultValue(entity);
      if (StringUtils.isBlank(entity.getId())) {
         entity.setId(this.keyGenerateService.keyGenerate());
      }

      if (!this.checkData(entity)) {
         throw new BasicServiceException("[" + this.modelClazz.getSimpleName() + "]校验不通过，请检查数据！");
      } else {
         this.getBasicMapper().insert(entity);
         this.afterInsert(entity);
         this.afterUpsert(entity);
      }
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void batchInsert(List<T> list) throws Exception {
      for(int i = 0; list != null && i < list.size(); ++i) {
         this.insert((BasicModel)list.get(i));
      }

   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void batchInsert(List<T> list, boolean useMyBatisBatch) throws Exception {
      if (useMyBatisBatch) {
         for(int i = 0; i < list.size(); ++i) {
            this.beforInsert((BasicModel)list.get(i));
            this.beforUpsert((BasicModel)list.get(i));
            ((BasicModel)list.get(i)).setId(this.keyGenerateService.keyGenerate());
         }

         this.getBasicMapper().batchInsert(list);
      } else {
         this.batchInsert(list);
      }

   }

   public void batchUpsert(List<T> list) throws Exception {
      for(int i = 0; list != null && i < list.size(); ++i) {
         this.upsert((BasicModel)list.get(i));
      }

   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public T preDefaultValue(T entity) throws Exception {
      T newEntity = (BasicModel)this.modelClazz.newInstance();
      newEntity.setId(this.keyGenerateService.keyGenerate());
      return newEntity;
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public T postDefaultValue(T entity) throws Exception {
      return entity;
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void upsert(T entity) throws Exception {
      if (this.isInsertFlag(entity)) {
         this.insert(entity);
      } else {
         this.update(entity);
      }

   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void update(T entity) throws Exception {
      this.beforUpdate(entity);
      this.beforUpsert(entity);
      if (!this.checkData(entity)) {
         throw new BasicServiceException("[" + this.modelClazz.getSimpleName() + "]校验不通过，请检查数据！");
      } else {
         this.checkId(entity);
         Integer effectiveRows = this.getBasicMapper().update(entity);
         boolean checkUpdate = CheckUpdAndDelUtil.checkUpdateOrDeleteVersion(effectiveRows, entity, this, "update");
         if (!checkUpdate) {
            throw new BasicServiceException("当前记录已经被其他人更新，请重新查询数据后重试！");
         } else {
            this.afterUpdate(entity);
            this.afterUpsert(entity);
         }
      }
   }

   public T queryById(T entity) throws Exception {
      this.checkId(entity);
      return this.getBasicMapper().queryById(entity);
   }

   public T queryById(String entityId) throws Exception {
      T entity = (BasicModel)this.modelClazz.newInstance();
      entity.setId(entityId);
      return this.queryById(entity);
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void deleteById(T entity) throws Exception {
      this.beforDelete(entity);
      this.checkId(entity);
      Integer effectiveRows = this.getBasicMapper().deleteById(entity);
      boolean checkDelete = CheckUpdAndDelUtil.checkUpdateOrDeleteVersion(effectiveRows, entity, this, "delete");
      if (!checkDelete) {
         throw new BasicServiceException("当前记录已经被其他人更新，请重新查询数据后重试！");
      } else {
         this.afterDelete(entity);
      }
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void deleteById(String entityId) throws Exception {
      T entity = (BasicModel)this.modelClazz.newInstance();
      entity.setId(entityId);
      this.deleteById(entity);
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void batchDelete(List<T> list) throws Exception {
      for(int i = 0; list != null && i < list.size(); ++i) {
         this.deleteById((BasicModel)list.get(i));
      }

   }

   public List<T> queryAll(T entity) throws Exception {
      this.beforQueryAll(entity);
      this.beforQueryAllOrExample(entity);
      return this.getBasicMapper().queryAll(entity);
   }

   public List<T> queryByExamplePage(T entity) throws Exception {
      this.beforQueryExample(entity);
      this.beforQueryAllOrExample(entity);
      List<T> list = this.getBasicMapper().queryByExamplePage(entity);
      this.afterQuery(entity, list);
      return list;
   }

   public List<T> queryAllByExamplePage(T entity) throws Exception {
      this.beforQueryExample(entity);
      this.beforQueryAllOrExample(entity);
      List<T> retList = new ArrayList();
      Boolean pageFlag = entity.getPageFlag();
      if (pageFlag) {
         return this.getBasicMapper().queryByExamplePage(entity);
      } else {
         while(true) {
            List<T> retOne = this.getBasicMapper().queryByExamplePage(entity);
            if (retOne.isEmpty()) {
               return retList;
            }

            retList.addAll(retOne);
            int curPage = entity.getPage() + 1;
            entity.setPage(curPage);
         }
      }
   }

   public boolean isInsertFlag(T entity) throws Exception {
      String statusFlag = entity.getRow_status();
      if (StringUtils.equals(statusFlag, "NEW")) {
         return true;
      } else if (StringUtils.equals(statusFlag, "UPDATE")) {
         return false;
      } else {
         throw new BasicServiceException("[" + this.modelClazz.getSimpleName() + "]无法识别当前操作状态。");
      }
   }

   public boolean checkData(T entity) throws Exception {
      return true;
   }

   public void beforQueryAllOrExample(T entity) throws Exception {
   }

   public void beforStreamQueryAll(T entity) throws Exception {
   }

   public void afterStreamQueryAllMulti(List<T> list) throws Exception {
   }

   public void beforQueryAll(T entity) throws Exception {
   }

   public void beforQueryExample(T entity) throws Exception {
   }

   public void beforUpdate(T entity) throws Exception {
   }

   public void afterUpdate(T entity) throws Exception {
   }

   public void beforInsert(T entity) throws Exception {
   }

   public void afterInsert(T entity) throws Exception {
   }

   public void beforDelete(T entity) throws Exception {
   }

   public void beforUpsert(T entity) throws Exception {
   }

   public void afterQuery(T entity, List<T> list) throws Exception {
   }

   public void afterUpsert(T entity) throws Exception {
   }

   public void afterDelete(T entity) throws Exception {
   }

   public boolean isExistedById(T entity) throws Exception {
      if (null != entity && null != entity.getId()) {
         entity.getFilterList().add(new Filter("idFilter", "id", "=", entity.getId().toString()));
         List<T> resultList = this.getBasicMapper().queryByExamplePage(entity);
         return null != resultList && !resultList.isEmpty();
      } else {
         throw new BasicServiceException("isExistedById() -> 未获取到参数id");
      }
   }

   public T queryOneByProperty(T entity, String propertyName, String operator) throws Exception {
      if (StringUtils.isBlank(propertyName)) {
         return null;
      } else {
         if (StringUtils.isBlank(operator)) {
            operator = "=";
         }

         entity.setPageFlag(true);
         entity.setPage(1);
         entity.setRows(1);
         List<Field> allFields = ReflectUtil.getAllFields(entity.getClass());

         for(int i = 0; i < allFields.size(); ++i) {
            Field field = (Field)allFields.get(i);
            if (propertyName.equals(field.getName())) {
               field.setAccessible(true);
               Object value = field.get(entity);
               List<Filter> filters = new ArrayList(2);
               filters.add(new Filter(propertyName, propertyName, operator, value.toString()));
               entity.getQps().setFilters(filters);
               break;
            }
         }

         List<T> resultList = this.getBasicMapper().queryByExamplePage(entity);
         return null != resultList && !resultList.isEmpty() ? (BasicModel)resultList.get(0) : null;
      }
   }

   public void updateSpecifiedFields(T entity, String... fieldNames) throws Exception {
      if (null != fieldNames) {
         StringBuilder sb = new StringBuilder("");

         for(int i = 0; i < fieldNames.length; ++i) {
            sb.append(",");
            sb.append(fieldNames);
         }

         String result = sb.toString();
         entity.setUpdateFields(result.substring(1, result.length()));
      }

      this.update(entity);
   }

   public void checkId(T entity) throws Exception {
      if (entity == null) {
         throw new BasicServiceException(this.modelClazz.getSimpleName() + "不能为空！");
      } else if (null == entity.getId()) {
         throw new BasicServiceException(this.modelClazz.getSimpleName() + "的ID不能为空！");
      }
   }

   public void batchUpdate(List<T> list) throws Exception {
      if (list != null && list.size() != 0) {
         Iterator var2 = list.iterator();

         while(var2.hasNext()) {
            T record = (BasicModel)var2.next();
            this.update(record);
         }

      }
   }

   private Class<T> getEntityClazz(Class<?> clazz) {
      if (clazz == null) {
         return null;
      } else if (Object.class == clazz) {
         return null;
      } else {
         Type genericSuperclass = clazz.getGenericSuperclass();
         if (genericSuperclass instanceof ParameterizedType) {
            ParameterizedType parameterizedType = (ParameterizedType)genericSuperclass;
            Type[] actualTypeArguments = parameterizedType.getActualTypeArguments();
            return (Class)actualTypeArguments[0];
         } else {
            boolean isFather = BasicServiceImpl.class.isAssignableFrom(clazz);
            if (isFather) {
               Class<?> superClass = clazz.getSuperclass();
               return this.getEntityClazz(superClass);
            } else {
               return null;
            }
         }
      }
   }

   public void streamQueryExport(T entity, ExportWriter exportWriter) throws Exception {
      this.beforQueryExample(entity);
      this.beforQueryAllOrExample(entity);
      exportWriter.writeData(entity, this.getBasicMapper());
   }

   public void exportCsv(T entity, List<ExportColumn> exportColumns, CsvWriter writer) throws Exception {
      this.beforQueryExample(entity);
      this.beforQueryAllOrExample(entity);
      int count = 0;

      try {
         LinkStdCsvStreamResultHandler linkStdCsvStreamResultHandler = new LinkStdCsvStreamResultHandler(exportColumns, writer, count, this);
         this.getBasicMapper().streamQueryAll(entity, linkStdCsvStreamResultHandler);
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("流式查询导出csv出现异常：" + e.getMessage(), e);
      } finally {
         writer.close();
      }

   }

   public void exportExcel(T entity, List<ExportColumn> exportColumns, ExcelWriter excelWriter, WriteSheet sheet, AbstractFileWriter fileWriter) throws Exception {
      this.beforQueryExample(entity);
      this.beforQueryAllOrExample(entity);
      int count = 0;

      try {
         LinkStdExcelStreamResultHandler linkStdExcelStreamResultHandler = new LinkStdExcelStreamResultHandler(exportColumns, excelWriter, sheet, count, this, fileWriter);
         this.getBasicMapper().streamQueryAll(entity, linkStdExcelStreamResultHandler);
      } catch (Exception var11) {
         Exception e = var11;
         logger.error("流式查询导出excel出现异常：" + e.getMessage(), e);
      } finally {
         excelWriter.finish();
      }

   }

   public void afterStreamQueryAll(T exportData) throws Exception {
   }
}
